# -*- encoding: utf-8 -*-
##############################################################################
#    
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.     
#
##############################################################################


from osv import osv, fields
import time
import dateutil
import dateutil.parser
from dateutil.relativedelta import relativedelta
from datetime import datetime, date
from tools.translate import _
from tools import DEFAULT_SERVER_DATE_FORMAT, DEFAULT_SERVER_DATETIME_FORMAT, float_compare
import decimal_precision as dp
import netsvc
import openerp
import calendar
from openerp import SUPERUSER_ID
import pytz


class stock_invoice_onshipping(osv.osv_memory):
    _name = 'stock.invoice.onshipping'
    _inherit ='stock.invoice.onshipping'
    _columns = {
        'groupby_product_in': fields.boolean('Agrupar Productos', 
            help='Si activa esta opcion se agruparan las lineas de los albaranes a Facturar.'),
        'type_in': fields.selection([('out', 'Sending Goods'), ('in', 'Getting Goods'), ('internal', 'Internal')], 'Shipping Type', required=True, select=True, help="Shipping type specify, goods coming in or going out."),
        }

    def _get_type_in(self, cr, uid, context=None):
        type_p = ""
        active_ids = context and context.get('active_ids', False)
        model = context and context.get('active_model', False)
        if model:
            model_obj = self.pool.get(model)
            picking_br = model_obj.browse(cr, uid, active_ids, context=None)[0]
            type_p = picking_br.type
        return type_p

    _defaults = {
        'type_in': _get_type_in,
        }

    def open_invoice(self, cr, uid, ids, context=None):
        #print "nuevo desarrollo ok------------->>>>"
        self_br = self.browse(cr, uid, ids[0], context=None)
        result = {}
        active_ids = context.get('active_ids', [])
        ###### PRIMERO LAS VALIDACIONES ########
        cr.execute("""
            select name from stock_picking where state != 'done'
            and id in %s
            """, (tuple(active_ids),))
        cr_res = cr.fetchall()
        validation_list_ids = [str(x[0]) for x in cr_res]
        if validation_list_ids:
            raise osv.except_osv(_('Error!'),
                _('Los siguientes Movimientos: %s \n No pudieron ser Facturados, \
                    debes asegurarte que todos se encuentran Entregados.' % str(validation_list_ids) ))

        if self_br.groupby_product_in == True and self_br.group == False:
            raise osv.except_osv(_('Error, creacion Factura!'),
                _('Solo puedes agrupar por Producto si agrupas por Cliente.' ))

        if self_br.groupby_product_in == False:
            result = super(stock_invoice_onshipping, self).open_invoice(cr, uid, ids, context)
            return result
        else:
            invoice_ids = []
            data_pool = self.pool.get('ir.model.data')
            res = self.create_invoice_grouped_product_in(cr, uid, ids, context=context)
            invoice_ids = res
            inv_type = context.get('inv_type', False)
            action_model = False
            action = {}
            if not invoice_ids:
                raise osv.except_osv(_('Error!'), _('Por favor, Crea Facturas.'))
            if inv_type == "out_invoice":
                action_model,action_id = data_pool.get_object_reference(cr, uid, 'account', "action_invoice_tree1")
            elif inv_type == "in_invoice":
                action_model,action_id = data_pool.get_object_reference(cr, uid, 'account', "action_invoice_tree2")
            elif inv_type == "out_refund":
                action_model,action_id = data_pool.get_object_reference(cr, uid, 'account', "action_invoice_tree3")
            elif inv_type == "in_refund":
                action_model,action_id = data_pool.get_object_reference(cr, uid, 'account', "action_invoice_tree4")
            if action_model:
                action_pool = self.pool.get(action_model)
                action = action_pool.read(cr, uid, action_id, context=context)
                action['domain'] = "[('id','in', ["+','.join(map(str,invoice_ids))+"])]"

            return action
        return result

    def create_invoice_grouped_product_in(self, cr, uid, ids, context=None):
        #print "nuevo desarrollo------------->>>>>"
        if context is None:
            context = {}
        self_br = self.browse(cr, uid, ids[0], context=None)
        picking_pool = self.pool.get('stock.picking')
        onshipdata_obj = self.read(cr, uid, ids, ['journal_id', 'group', 'invoice_date'])
        if context.get('new_picking', False):
            onshipdata_obj['id'] = onshipdata_obj.new_picking
            onshipdata_obj[ids] = onshipdata_obj.new_picking
        context['date_inv'] = onshipdata_obj[0]['invoice_date']
        active_ids = context.get('active_ids', [])
        active_picking = picking_pool.browse(cr, uid, context.get('active_id',False), context=context)
        inv_type = picking_pool._get_invoice_type(active_picking)
        context['inv_type'] = inv_type
        if isinstance(onshipdata_obj[0]['journal_id'], tuple):
            onshipdata_obj[0]['journal_id'] = onshipdata_obj[0]['journal_id'][0]
        res = picking_pool.action_invoice_create_grouped_product_in(cr, uid, active_ids,
              journal_id = onshipdata_obj[0]['journal_id'],
              group = onshipdata_obj[0]['group'],
              type = inv_type,
              invoice_date = self_br.invoice_date,
              context=context)
        return res


class stock_picking(osv.osv):
    _name = 'stock.picking'
    _inherit ='stock.picking'
    _columns = {
        }

    _defaults = {
        }

    def action_invoice_create_grouped_product_in(self, cr, uid, ids, journal_id=False,
            group=False, type='in_invoice', invoice_date=False, context=None):
        #print "nuevo desarrollo para agrupar albaranes"

        """ Creates invoice based on the invoice state selected for picking.
        @param journal_id: Id of journal
        @param group: Whether to create a group invoice or not
        @param type: Type invoice to be created
        @return: Ids of created invoices for the pickings
        """
        ############# VALIDAR POR PRECIO, PRODUCTO, REF_LINE
        if context is None:
            context = {}

        invoice_obj = self.pool.get('account.invoice')
        invoice_line_obj = self.pool.get('account.invoice.line')
        order_line_obj = self.pool.get('purchase.order.line')
        partner_obj = self.pool.get('res.partner')
        sale_obj = self.pool.get("purchase.order")
        invoices_group = {}
        res = {}
        inv_type = type
        
        invoice_created_ids = []
        #print "ids------->>>",ids
        #print "active ids",context
        ###### PRIMERO LAS VALIDACIONES ########
        cr.execute("""
            select name from stock_picking where invoice_state != '2binvoiced'
            and id in %s
            """, (tuple(ids),))
        cr_res = cr.fetchall()
        validation_list_ids = [str(x[0]) for x in cr_res]
        if validation_list_ids:
            raise osv.except_osv(_('Error!'),
                _('Los siguientes Movimientos: %s \n No pudieron ser Facturados, \
                    esto puede ser debido a que estan Facturados o son Albaranes no Facturables.' % str(validation_list_ids) ))

        # cr.execute("""
        #     select name from stock_picking where state != 'done'
        #     and id in %s
        #     """, (tuple(ids),))
        # cr_res = cr.fetchall()
        # validation_list_ids = [str(x[0]) for x in cr_res]
        # if validation_list_ids:
        #     raise osv.except_osv(_('Error!'),
        #         _('Los siguientes Movimientos: %s \n No pudieron ser Facturados, \
        #             debes asegurarte que todos se encuentran Entregados.' % str(validation_list_ids) ))


        product_send_list = []
        sale_order_ids = []
        for picking in self.browse(cr, uid, ids, context=context):
            if picking.purchase_id:
                sale_order_ids.append(picking.purchase_id.id)
            else:
                origin = picking.origin
                sale_ids = False
                if ':' in origin:
                    origin = tuple(origin.split(':'))
                    sale_ids = sale_obj.search(cr, uid, [('name','in',origin),('order_policy', '=', 'picking')])
                    sale_order_ids = sale_order_ids+sale_ids
                elif '|':
                    origin = origin.replace(' ','')
                    origin = tuple(origin.split('|'))
                    sale_ids = sale_obj.search(cr, uid, [('name','in',origin),('order_policy', '=', 'picking')])
                    sale_order_ids = sale_order_ids+sale_ids
                else:
                    sale_ids = sale_obj.search(cr, uid, [('name','=',origin),('order_policy', '=', 'picking')])
                    sale_order_ids = sale_order_ids+sale_ids
        partner_ids = []
        if sale_order_ids:
            cr.execute("""select partner_id from purchase_order 
                            where id in %s""",(tuple(sale_order_ids),))
            cr_res = cr.fetchall()
            partner_ids = [x[0] for x in cr_res]
            partner_ids = list(set(partner_ids))
        for partner in partner_ids:
            cr.execute("""select pr.currency_id from purchase_order so join product_pricelist pr 
                on so.pricelist_id = pr.id  where so.id in %s;""", (tuple(sale_order_ids),))
            cr_res = cr.fetchall()
            validate_list = [x[0] for x in cr_res if x[0]]
            validate_list = list(set(validate_list))
            if len(validate_list) > 1:
                raise osv.except_osv(
                _('Error !'),
                _('No puedes Facturar pedidos con distintas Monedas.' ))
            
            ################## PARTNER >>>>> partner
            ################## VALIDACIONES >>>>> validate_list

            order_ids = sale_obj.search(cr, uid,[('partner_id','=',partner),
                ('id','in',tuple(sale_order_ids))])
            ########## ORDER IDS >>>>>> order_ids
            order = sale_obj.browse(cr, uid, order_ids, context=None)[0]
            origin = []
            a = order.partner_id.property_account_payable.id
            if order.partner_id and order.partner_id.property_payment_term.id:
                pay_term = order.partner_id.property_payment_term.id
            else:
                pay_term = False
            for so in sale_obj.browse(cr, uid, order_ids, context=None):
                origin.append(str(so.name))
            # add_part = self.pool.get('res.partner').address_get(cr, uid, [order.partner_id.id], ['contact', 'invoice'])
            # contact_addr_id = add_part['contact']
            # address_invoice_id = add_part['invoice']
            cr.execute("""
                select id from res_partner where parent_id = %s ;
                """ % partner)
            cr_res = cr.fetchall()
            partner_list_ids = [x[0] for x in cr_res]
            partner_list_ids.append(partner)
            cr.execute("""
                select asn from stock_picking where id in %s and partner_id in %s ;
                """, (tuple(ids), tuple(partner_list_ids), ))
            cr_res = cr.fetchall()
            asn_list_ids = [str(x[0]) for x in cr_res if x]

            inv = {
                'name': '',
                'origin': str(origin).replace(' ','').replace(',',' | ').replace('[','').replace(']','').replace("'",''),
                'type': 'in_invoice',
                'reference': "P%dSO%d" % (order.partner_id.id, order.id),
                'account_id': a,
                'partner_id': order.partner_id.id,
                # 'invoice_line': [(6, 0, lines)],
                'currency_id' : order.pricelist_id.currency_id.id,
                'comment': order.notes,
                'payment_term': pay_term,
                # 'pay_method_id': order.pay_method_id.id if order.pay_method_id else False,
                'fiscal_position': order.fiscal_position.id or order.partner_id.property_account_position.id,
                #'user_id': order.user_id and order.user_id.id or False,
                'company_id': order.company_id and order.company_id.id or False,
                'date_invoice': fields.date.today() if invoice_date == False else invoice_date,
                'journal_id': journal_id,
                # 'address_invoice_id': address_invoice_id if address_invoice_id else contact_addr_id,
                # 'address_contact_id': contact_addr_id if contact_addr_id else address_invoice_id,
                'asn': asn_list_ids[0] if asn_list_ids else False,
                #'orden_compra': order.orden_compra,
            }
            ############### INV VALS >>>>>>>>>>> inv
            inv_id = self.pool.get('account.invoice').create(cr, uid, inv, context=None)
            ############### INV ID >>>>>>>>>>> inv_id
            invoice_created_ids.append(inv_id)
            ### PRODUCTOS ###
            cr.execute("""
                select product_id from purchase_order_line where order_id in %s
                """, (tuple(order_ids),))
            cr_res = cr.fetchall()
            product_ids = [x[0] for x in cr_res if x[0]]
            product_ids = list(set(product_ids))
            for product in product_ids:
                #print "entre a los productos",product
                #else:
                if product:
                    cr.execute("""
                        select price_unit from purchase_order_line where order_id in %s
                        and product_id = %s
                        """, (tuple(order_ids), product, ))
                    cr_res = cr.fetchall()
                    prices = [x[0] for x in cr_res if x[0]]
                    prices = list(set(prices))
                    ########## PRICES >>>>>>>> prices
                    #print "entre de los precios",prices
                    for price in prices:
                        #print "entre a los precios",price
                        stock_obj = self.pool.get("stock.move")
                        ids_lines=stock_obj.search(cr, uid, [('picking_id','in',tuple(ids))])

                        #print "ids_lines",ids_lines

                        order_line_ids = order_line_obj.search(cr, uid,
                            [('product_id','=', product),
                            ('price_unit','=',price),
                            ('order_id','in',tuple(order_ids))])
                        ############## ORDER LINE IDS >>> order_line_ids
                        
                        print "order_line_ids",order_line_ids
                        
                        if order_line_ids:
                            #print "entre al order_line_ids"
                            cr.execute("""select sum(product_qty) from stock_move 
                                    where picking_id in %s and purchase_line_id in %s and product_id=%s""", (tuple(ids),tuple(order_line_ids), product, ))
                            # cr.execute("""select sum(product_uom_qty) from sale_order_line 
                            #     where id in %s""", (tuple(order_line_ids),))
                            cr_res = cr.fetchall()

                            if cr_res and cr_res[0]:
                                product_uom_qty = cr_res[0][0]
                            else:
                                product_uom_qty = 0.0

                            #print "product_uom_qty",product_uom_qty

                            line_br = order_line_obj.browse(cr, uid, order_line_ids, context=None)[0]
                            #print "line_br",line_br
                            if line_br.product_id:
                                #print "entre linebr",line_br.product_id
                                if line_br.product_id.type == "product":
                                    account_id=line_br.product_id.categ_id.property_stock_account_input_categ.id
                                else:
                                    account_id=line_br.product_id.categ_id.property_account_expense_categ.id
                                    if line_br.product_id.valuation == "real_time":
                                        account_id= line_br.product_id.categ_id.property_stock_account_input_categ.id
                                # account_id = line_br.product_id.property_account_expense.id
                                # if not account_id:
                                #     account_id = line_br.product_id.categ_id.property_account_expense_categ.id
                                if not account_id:
                                    raise osv.except_osv(_('Error !'),
                                            _('No existe Cuenta Definida para el Producto: "%s" (id:%d)') % \
                                                (line_br.product_id.name, line_br.product_id.id,))

                            
                            if product_uom_qty > 0.0:
                                #print "generacion de la linea"
                                line_vals = {
                                    #'ref_line': line_br.ref_line,
                                    'name': line_br.name,
                                    'origin': line_br.order_id.name,
                                    'account_id': account_id,
                                    'price_unit': price,
                                    'quantity': product_uom_qty,
                                    #'discount': line_br.discount,
                                    'uos_id': line_br.product_uom.id,
                                    'product_id': line_br.product_id.id or False,
                                    'invoice_line_tax_id': [(6, 0, [x.id for x in line_br.taxes_id])],
                                    # 'note': line_br.notes,
                                    'account_analytic_id': line_br.order_id.proyect and line_br.order_id.proyect or False,
                                    'invoice_id': inv_id,
                                    }
                                line_created = self.pool.get('account.invoice.line').create(cr, uid, line_vals, context=None)
                                #print "linea creada",line_created
                                order_line_obj.write(cr, uid, order_line_ids, 
                                    {'invoice_lines': [(4, line_created)],'invoiced': True}, context=context) 

            sale_obj.write(cr, uid, order_ids, {'invoice_ids': [(4, inv_id)],'invoiced':True, 'invoiced_mark':True}, context=context) 
            ######### Al final Entonces Ignoramos Excepcion de Factura ######
            wf_service = netsvc.LocalService("workflow")
            for p_order in order_ids:
                o = sale_obj.browse(cr, uid, p_order, context=None)
                # if o.order_policy == 'picking':
                #     sale_obj.write(cr, uid, map(lambda x: x.id, o.picking_ids), {'invoice_state': 'invoiced'})
                sale_obj.write(cr, uid, [p_order], {'state': 'approved'})
                wf_service.trg_validate(uid, 'purchase.order', p_order, 'manual_invoice', cr)
                wf_service.trg_validate(uid, 'purchase.order', p_order, 'invoice_corrected', cr)
        ########## Marcando los Picking como Facturados 
        self.write(cr, uid, ids, {'invoice_state': 'invoiced',}, context=None)
        if invoice_created_ids:
            for invoice in invoice_created_ids:
                self.pool.get('account.invoice').button_reset_taxes(cr, uid, [invoice], context=context)
            return invoice_created_ids

        # raise osv.except_osv(_('Cherman!'),
        #             _('Interrupcion =).'))

        return []

